@@ -20,7 +20,7 @@ class TestStylesheet:
         parser = Stylesheet()
         xml = tostring(parser.to_tree())
         expected = """
-        <styleSheet>
+        <styleSheet xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main">
           <numFmts count="0"></numFmts>
           <cellStyleXfs count="0"></cellStyleXfs>
           <cellXfs count="0"></cellXfs>
@@ -145,25 +145,35 @@ class TestStylesheet:
 
     def test_custom_number_formats(self, Stylesheet, datadir):
         datadir.chdir()
-        import codecs
-        with codecs.open("styles_number_formats.xml", encoding="utf-8") as src:
-            xml = src.read().encode("utf8") # Python 2.6, Windows
-
+        with open("styles_number_formats.xml", "rb") as src:
+            xml = src.read()
         node = fromstring(xml)
         stylesheet = Stylesheet.from_tree(node)
 
-        assert stylesheet.number_formats == [
+        assert set(stylesheet.number_formats) == set([
             '_ * #,##0.00_ ;_ * \-#,##0.00_ ;_ * "-"??_ ;_ @_ ',
             "#,##0.00_ ",
             "yyyy/m/d;@",
             "0.00000_ "
-        ]
+        ])
+
+
+    def test_remove_duplicate_number_formats(self, Stylesheet, datadir):
+        datadir.chdir()
+
+        with open("builtins_as_custom_number_formats.xml", "rb") as src:
+            xml = src.read()
+            node = fromstring(xml)
+
+        stylesheet = Stylesheet.from_tree(node)
+
+        assert stylesheet.number_formats == ['dd\\/mm']
 
 
     def test_assign_number_formats(self, Stylesheet):
 
         node = fromstring("""
-        <styleSheet>
+        <styleSheet xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main">
         <numFmts count="1">
           <numFmt numFmtId="43" formatCode='_ * #,##0.00_ ;_ * \-#,##0.00_ ;_ * "-"??_ ;_ @_ ' />
         </numFmts>
@@ -194,6 +204,7 @@ class TestStylesheet:
         stylesheet = Stylesheet.from_tree(node)
 
         followed = stylesheet.named_styles['Followed Hyperlink']
+        assert followed.xfId == 2
         assert followed.name == "Followed Hyperlink"
         assert followed.font == stylesheet.fonts[2]
         assert followed.fill == DEFAULT_EMPTY_FILL
