@@ -1,5 +1,5 @@
 from __future__ import absolute_import
-# Copyright (c) 2010-2016 openpyxl
+# Copyright (c) 2010-2017 openpyxl
 
 """
 OOXML has non-standard escaping for characters < \031
@@ -10,7 +10,7 @@ import re
 
 def escape(value):
     r"""
-    Convert ASCII < 31 to OOXML: \n == _x + hex(ord(\n)) +_
+    Convert ASCII < 31 to OOXML: \n == _x + hex(ord(\n)) + _
     """
 
     CHAR_REGEX = re.compile(r"[\001-\031]")
@@ -19,7 +19,7 @@ def escape(value):
         """
         Callback to escape chars
         """
-        return "_x%04x_" % ord(match.group(0)) # py 2.6
+        return "_x{:0>4x}_".format(ord(match.group(0)))
 
     return CHAR_REGEX.sub(_sub, value)
 
