@@ -7,17 +7,18 @@ from openpyxl.tests.helper import compare_xml
 
 @pytest.fixture
 def PivotCache():
-    from ..pivot import PivotCache
+    from ..parser import PivotCache
     return PivotCache
 
 
 class TestPivotCache:
 
     def test_ctor(self, PivotCache):
-        pivot = PivotCache(1)
+        pivot = PivotCache(cacheId=1, id="rId1")
         xml = tostring(pivot.to_tree())
         expected = """
-        <pivotCache cacheId="1" />
+        <pivotCache cacheId="1" r:id="rId1"
+          xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships"/>
         """
         diff = compare_xml(xml, expected)
         assert diff is None, diff
@@ -30,30 +31,3 @@ class TestPivotCache:
         node = fromstring(src)
         pivot = PivotCache.from_tree(node)
         assert pivot == PivotCache(2)
-
-
-@pytest.fixture
-def PivotCacheList():
-    from ..pivot import PivotCacheList
-    return PivotCacheList
-
-
-class TestPivotCacheList:
-
-    def test_ctor(self, PivotCacheList):
-        pivot = PivotCacheList()
-        xml = tostring(pivot.to_tree())
-        expected = """
-        <pivotCaches />
-        """
-        diff = compare_xml(xml, expected)
-        assert diff is None, diff
-
-
-    def test_from_xml(self, PivotCacheList):
-        src = """
-        <pivotCaches />
-        """
-        node = fromstring(src)
-        pivot = PivotCacheList.from_tree(node)
-        assert pivot == PivotCacheList()
