@@ -102,12 +102,25 @@ def test_getitem(Workbook, Worksheet):
         wb['NotThere']
 
 
-def test_delitem(Workbook):
+def test_get_chartsheet(Workbook):
+    wb = Workbook()
+    cs = wb.create_chartsheet()
+    assert wb[cs.title] is cs
+
+
+def test_del_worksheet(Workbook):
     wb = Workbook()
     del wb['Sheet']
     assert wb.worksheets == []
 
 
+def test_del_chartsheet(Workbook):
+    wb = Workbook()
+    cs = wb.create_chartsheet()
+    del wb[cs.title]
+    assert wb.chartsheets == []
+
+
 def test_contains(Workbook):
     wb = Workbook()
     assert "Sheet" in wb
@@ -165,6 +178,14 @@ def test_remove_named_range():
     assert 'test_nr' not in named_ranges_list
 
 
+def test_remove_sheet_with_names():
+    wb = Workbook()
+    new_sheet = wb.create_sheet()
+    wb.create_named_range('test_nr', new_sheet, 'A1', 1)
+    del wb['Sheet1']
+    assert wb.defined_names.definedName == []
+
+
 def test_add_invalid_worksheet_class_instance():
 
     class AlternativeWorksheet(object):
@@ -190,14 +211,18 @@ class TestCopy:
         assert ws2 is not None
 
 
-    def test_worksheet_copy_name(self):
+    @pytest.mark.parametrize("title, copy",
+                             [
+                                 ("TestSheet", "TestSheet Copy"),
+                                 (u"D\xfcsseldorf", u"D\xfcsseldorf Copy")
+                                 ]
+                             )
+    def test_worksheet_copy_name(self, title, copy):
         wb = Workbook()
         ws1 = wb.active
-        ws1.title = "TestSheet"
+        ws1.title = title
         ws2 = wb.copy_worksheet(ws1)
-        ws3 = wb.copy_worksheet(ws1)
-        assert ws2.title == 'TestSheet Copy'
-        assert ws3.title == 'TestSheet Copy1'
+        assert ws2.title == copy
 
 
     def test_cannot_copy_readonly(self):
