@@ -7,6 +7,8 @@ from openpyxl.compat import safe_string
 from openpyxl.comments.comment_sheet import CommentRecord
 from openpyxl.xml.functions import Element, SubElement
 from openpyxl import LXML
+from openpyxl.utils.datetime import to_excel, days_to_time
+from datetime import timedelta
 
 
 def get_rows_to_write(worksheet):
@@ -68,6 +70,15 @@ def etree_write_cell(xf, worksheet, cell, styled=None):
 
     value = cell._value
 
+    if cell.data_type == "d":
+        if cell.parent.parent.iso_dates:
+            if isinstance(value, timedelta):
+                value = days_to_time(value)
+            value = value.isoformat()
+        else:
+            attributes['t'] = "n"
+            value = to_excel(value)
+
     if cell._comment is not None:
         comment = CommentRecord.from_cell(cell)
         worksheet._comments.append(comment)
@@ -107,6 +118,15 @@ def lxml_write_cell(xf, worksheet, cell, styled=False):
 
     value = cell._value
 
+    if cell.data_type == "d":
+        if cell.parent.parent.iso_dates:
+            if isinstance(value, timedelta):
+                value = days_to_time(value)
+            value = value.isoformat()
+        else:
+            attributes['t'] = "n"
+            value = to_excel(value)
+
     if cell._comment is not None:
         comment = CommentRecord.from_cell(cell)
         worksheet._comments.append(comment)
