@@ -28,7 +28,7 @@ def test_write_auto_filter(datadir):
     datadir.chdir()
     wb = Workbook()
     ws = wb.active
-    ws.cell('F42').value = 'hello'
+    ws['F42'].value = 'hello'
     ws.auto_filter.ref = 'A1:F1'
 
     content = write_workbook(wb)
@@ -46,6 +46,7 @@ def test_write_hidden_worksheet():
     expected = """
     <workbook xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships">
     <workbookPr/>
+    <workbookProtection/>
     <bookViews>
       <workbookView activeTab="1"/>
     </bookViews>
@@ -115,12 +116,13 @@ def test_write_workbook(datadir):
 def test_write_named_range():
     wb = Workbook()
     ws = wb.active
+    ws.title = "Test Sheet"
     wb.create_named_range("test_range", ws, value="A1:B5")
 
     xml = tostring(wb.defined_names.to_tree())
     expected = """
     <definedNames>
-     <definedName name="test_range">Sheet!A1:B5</definedName>
+     <definedName name="test_range">'Test Sheet'!A1:B5</definedName>
     </definedNames>
     """
     diff = compare_xml(xml, expected)
@@ -135,6 +137,7 @@ def test_write_workbook_code_name():
     expected = """
     <workbook xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships">
     <workbookPr codeName="MyWB"/>
+    <workbookProtection/>
     <bookViews>
       <workbookView activeTab="0"/>
     </bookViews>
@@ -165,6 +168,20 @@ def test_write_root_rels():
     assert diff is None, diff
 
 
+def test_write_workbook_protection(datadir):
+    from ...workbook.protection import WorkbookProtection
+
+    datadir.chdir()
+    wb = Workbook()
+    wb.security = WorkbookProtection(lockStructure=True)
+    wb.security.set_workbook_password('ABCD', already_hashed=True)
+
+    content = write_workbook(wb)
+    with open('workbook_protection.xml') as expected:
+        diff = compare_xml(content, expected.read())
+        assert diff is None, diff
+
+
 @pytest.fixture
 def Unicode_Workbook():
     wb = Workbook()
@@ -183,6 +200,7 @@ def test_print_area(Unicode_Workbook):
     <workbook xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships"
     xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main">
     <workbookPr/>
+    <workbookProtection/>
     <bookViews>
       <workbookView activeTab="0"/>
     </bookViews>
@@ -209,6 +227,7 @@ def test_print_titles(Unicode_Workbook):
     <workbook xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships"
     xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main">
     <workbookPr/>
+    <workbookProtection/>
     <bookViews>
       <workbookView activeTab="0"/>
     </bookViews>
@@ -238,6 +257,7 @@ def test_print_autofilter(Unicode_Workbook):
     <workbook xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships"
     xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main">
     <workbookPr/>
+    <workbookProtection/>
     <bookViews>
       <workbookView activeTab="0"/>
     </bookViews>
